package dados.filial;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import dados.exception.EnderecoInvalidoException;

@Entity
@Table(name="Endereco")
public class Endereco implements Serializable {
	
	private static final long serialVersionUID = 5213836341901375218L;

	@Id @GeneratedValue()
	@Column(name="id_Endereco")
	private Long id;
	
	@Column(updatable=true, name="Logradouro", nullable=false)
	private String logradouro;

	@Column(updatable=true, name="Estado", nullable=false)
	private String estado;

	@Column(updatable=true, name="Cidade", nullable=false)
	private String cidade;

	@Column(updatable=true, name="Bairro", nullable=false)
	private String bairro;

	@Column(updatable=true, name="Complemento", nullable=true)
	private String complemento;

	@Column(updatable=true, name="Numero", nullable=false)
	private Integer numero;

	@Column(updatable=true, name="CEP", nullable=false)
	private Integer cep;
	
	public Endereco() {}
	
	public Endereco(String logradouro, String estado, String cidade,
			String bairro, String complemento, Integer numero, Integer cep) throws EnderecoInvalidoException {
		
		if(Endereco.isValido(logradouro, estado, cidade, bairro, complemento, numero, cep)){
			this.logradouro = logradouro.trim();
			this.estado = estado.trim();
			this.cidade = cidade.trim();
			this.bairro = bairro.trim();
			this.complemento = complemento;
			this.numero = numero;
			this.cep = cep;
			
			if(this.complemento != null){
				this.complemento = this.complemento.trim();
			}
		}else{
			throw new EnderecoInvalidoException();
		}
	}
	
	public Endereco(Long id, String logradouro, String estado, String cidade,
			String bairro, String complemento, Integer numero, Integer cep) throws EnderecoInvalidoException {
		this(logradouro, estado, cidade, bairro, complemento, numero, cep);
		
		if(id != null){
			this.id = id;
		}else{
			throw new EnderecoInvalidoException();
		}
	}
	
	public Long getID(){
		return this.id;
	}
	
	public void setID(Long id) throws EnderecoInvalidoException{
		if(id != null){
			this.id = id;
		}else{
			throw new EnderecoInvalidoException();
		}
	}
	
	public String getLogradouro() {
		return this.logradouro;
	}

	public void setLogradouro(String logradouro) {
		this.logradouro = logradouro;
	}

	public String getEstado() {
		return this.estado;
	}

	public void setEstado(String estado) {
		this.estado = estado;
	}

	public String getCidade() {
		return this.cidade;
	}

	public void setCidade(String cidade) {
		this.cidade = cidade;
	}

	public String getBairro() {
		return this.bairro;
	}

	public void setBairro(String bairro) {
		this.bairro = bairro;
	}

	public String getComplemento() {
		return this.complemento;
	}

	public void setComplemento(String complemento) {
		this.complemento = complemento;
	}

	public int getNumero() {
		return this.numero;
	}

	public void setNumero(int numero) {
		this.numero = numero;
	}

	public int getCep() {
		return this.cep;
	}

	public void setCep(int cep) {
		this.cep = cep;
	}
	
	public String formatarCEP(){
		String cep = Integer.toString(this.cep);
		String retorno = cep.substring(0, cep.length() - 3) + "-" + cep.substring(cep.length()-3);
		return retorno;
	}

	public static boolean isValido(String logradouro, String estado, String cidade,
			String bairro, String complemento, int numero, int cep){
		return logradouro != null && !logradouro.isEmpty() && estado != null && !estado.isEmpty()
		&& cidade != null && !cidade.isEmpty() && bairro != null && !bairro.isEmpty() && numero > 0 && cep > 0 && cep <= 99999999;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Endereco && this.cep == ((Endereco) obj).getCep();
	}
	
	@Override
	public String toString() {
		return String.format("%s, %d, %s, %s, %s, %s", this.logradouro, this.numero, this.bairro, this.formatarCEP(), this.cidade, this.estado);
	}
	
}
