package dados.filial;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.Table; 

import dados.exception.FilialInvalidaException;
import dados.usuario.Telefone;

@Entity
@Table(name="Filial")
public class Filial implements Serializable {
	
	private static final long serialVersionUID = 9048779389851180504L;

	@Id
	@Column(name="id_Filial")
	private String nome;
	
	@Column(name="id")
	@GeneratedValue
	private long id;
	
	@OneToOne(cascade = CascadeType.ALL, optional=false)
    @JoinColumn(name="id_Telefone")
	private Telefone telefone;

	@OneToOne(cascade = CascadeType.ALL, optional=false, fetch=FetchType.EAGER)
    @JoinColumn(name="id_Endereco")
	private Endereco endereco;
	
	public Filial() {}
	
	public Filial(String nome, Telefone telefone, Endereco e) throws FilialInvalidaException{
		if(Filial.isValido(telefone, nome, e)){
			this.nome = nome;
			this.endereco = e;
			this.telefone = telefone;
		}else{
			throw new FilialInvalidaException();
		}
	}
	
	public static boolean isValido(Telefone telefone, String nome, Endereco endereco){
		return telefone != null && nome != null && endereco != null; 
	}

	public Telefone getTelefone() {
		return this.telefone;
	}

	public void setTelefone(Telefone telefone) throws FilialInvalidaException {
		if(Filial.isValido(telefone, this.nome, this.endereco)){
			this.telefone = telefone;
		}else{
			throw new FilialInvalidaException();
		}
	}

	public String getNome() {
		return this.nome;
	}

	public void setNome(String nome) throws FilialInvalidaException {
		if(Filial.isValido(this.telefone, nome, this.endereco)){
			this.nome = nome;
		}else{
			throw new FilialInvalidaException();
		}
	}

	public Endereco getEndereco() {
		return this.endereco;
	}

	public void setEndereco(Endereco endereco) throws FilialInvalidaException {
		if(Filial.isValido(this.telefone, this.nome, endereco)){
			this.endereco = endereco;
		}else{
			throw new FilialInvalidaException();
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Filial && this.nome.equalsIgnoreCase(((Filial)obj).getNome());
	}

	public long getId() {
		return this.id;
	}

	public void setId(long id) {
		this.id = id;
	}
}
