package dados.locacao;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import dados.exception.LocacaoInvalidaException;
import dados.filial.Endereco;
import dados.usuario.Cliente;

@Entity
@Table(name="Locacao")
public class Locacao implements Serializable {
	
	private static final long serialVersionUID = -6506580943653305377L;
	
	@Id
	@Column(name="id")
	@GeneratedValue
	private long id;
	
	@OneToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, optional=false, fetch=FetchType.EAGER)
	@JoinColumn(name="id_Copia")
	private Copia copia;
	
	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, optional=false, fetch=FetchType.EAGER)
	@JoinColumn(name="id_Cliente")
	private Cliente cliente;

	@Embedded
	private Periodo periodo;
	
	@Column(name="locacao_eh_gratuita")
	boolean gratuita;
	
	@Column(name="custo")
	double custo;
	
	public Locacao() {}
	
	public Locacao(Cliente cliente, Copia copia, Periodo periodo) throws LocacaoInvalidaException {
		if(cliente != null && copia != null && periodo != null){
			this.copia = copia;
			this.cliente = cliente;
			this.periodo = periodo;
			this.gratuita = false;
			this.custo = 0;
		}else{
			throw new LocacaoInvalidaException();
		}
	}

	public Copia getCopia() {
		return this.copia;
	}

	public void setCopia(Copia copia) throws LocacaoInvalidaException {
		if(copia != null){
			this.copia = copia;
		}else{
			throw new LocacaoInvalidaException();
		}
	}

	public Cliente getCliente() {
		return this.cliente;
	}

	public void setCliente(Cliente cliente) throws LocacaoInvalidaException {
		if(cliente != null){
			this.cliente = cliente;
		}else{
			throw new LocacaoInvalidaException();
		}
	}

	public Periodo getPeriodo() {
		return this.periodo;
	}

	public void setPeriodo(Periodo periodo) throws LocacaoInvalidaException {
		if(periodo != null){
			this.periodo = periodo;
		}else{
			throw new LocacaoInvalidaException();
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Endereco && this.getCliente().equals(((Locacao)obj).getCliente()) && this.getCopia().equals(((Locacao)obj).getCopia());
	}

	public double getCusto() {
		if(this.gratuita){
			return this.custo;
		} else {
			return this.custo + this.periodo.getQuantidadeDias() * this.getCopia().getProduto().getClassificacaoPreco().getPreco();
		}
	}

	public boolean isGratuita() {
		return this.gratuita;
	}

	public void setGratuita(boolean gratuita) {
		this.gratuita = gratuita;
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
	
	public void setCusto(double custo){
		this.custo = custo;
	}
}
