package dados.locacao;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import dados.exception.PeriodoInvalidoException;
import dados.filial.Filial;

@Embeddable
public class Periodo implements Serializable {

	private static final long serialVersionUID = 4998804370608604064L;
	
	@Column(updatable=true, name="DataInicio", nullable=false)
	@Temporal(TemporalType.DATE)
	private Calendar dataInicio;
	
	@Column(updatable=true, name="DataFim", nullable=false)
	@Temporal(TemporalType.DATE)
	private Calendar dataFim;
	
	public Periodo() {}
	
	public Periodo(Calendar dataInicio, Calendar dataFim) throws PeriodoInvalidoException {
		if(dataInicio != null && dataFim != null && dataInicio.before(dataFim)){
			this.dataInicio = dataInicio;
			this.dataFim = dataFim;
		}else{
			throw new PeriodoInvalidoException(dataInicio, dataFim);
		}
	}

	public Calendar getDataInicio() {
		return this.dataInicio;
	}

	public void setDataInicio(Calendar dataInicio) throws PeriodoInvalidoException {
		if(dataInicio != null && dataFim != null && dataInicio.before(dataFim)){
			this.dataInicio = dataInicio;
		}else{
			throw new PeriodoInvalidoException(dataInicio, this.dataFim);
		}
	}

	public Calendar getDataFim() {
		return this.dataFim;
	}

	public void setDataFim(Calendar dataFim) throws PeriodoInvalidoException {
		if(dataInicio != null && dataFim != null && dataInicio.before(dataFim)){
			this.dataFim = dataFim;
		}else{
			throw new PeriodoInvalidoException(this.dataInicio, dataFim);
		}
	}
	
	public boolean intersecta(Periodo periodo){
		boolean retorno = (periodo != null);
		
		if(retorno){
			retorno = this.dataInicio.before(periodo.getDataFim()) && this.dataFim.after(periodo.getDataInicio());
			retorno = retorno || (this.dataFim.before(periodo.getDataFim()) && this.dataFim.after(periodo.getDataInicio()));
			retorno = retorno || (this.dataInicio.before(periodo.getDataFim()) && this.dataInicio.after(periodo.getDataInicio()));
			retorno = retorno || (this.dataInicio.before(periodo.getDataInicio()) && this.dataFim.after(periodo.getDataFim()));
			retorno = retorno || (this.dataFim.equals(periodo.getDataFim()) || this.dataInicio.equals(periodo.getDataInicio()));
		}
		
		return retorno;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Filial && this.dataInicio.equals(((Periodo)obj).getDataInicio()) && this.dataFim.equals(((Periodo)obj).getDataFim());
	}
	
	@Override
	public String toString() {
		SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
		return "Data Incio: " + format.format(this.dataInicio.getTime()) + "   Data Fim: " + format.format(this.dataFim.getTime());
	}

	public int getQuantidadeDias() {
		return (int) Math.ceil(((double) this.dataFim.getTimeInMillis() - this.dataInicio.getTimeInMillis()) / 1000*60*60*24);
	}
	
}
