package dados.locacao;

import java.io.Serializable;
import java.text.SimpleDateFormat;

import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Table;

import dados.exception.ReservaInvalidaException;
import dados.filial.Endereco;
import dados.usuario.Cliente;

@Entity
@Table(name="Reserva")
public class Reserva implements Serializable{

	private static final long serialVersionUID = 2975808736449617250L;

	@EmbeddedId
	private ClienteCopiaPK chave;
	
	@Column(name="id")
	@GeneratedValue
	private long id;
	
    @Embedded
	private Periodo periodo;
	
	public Reserva() {}
	
	public Reserva(Cliente cliente, Copia copia, Periodo periodo) throws ReservaInvalidaException {
		if(cliente != null && copia != null && periodo != null){
			this.chave = new ClienteCopiaPK(copia, cliente);
			this.periodo = periodo;
		}else{
			throw new ReservaInvalidaException(cliente, copia, periodo);
		}
	}

	public Copia getCopia() {
		return this.chave.getCopia();
	}
	
	public void setCopia(Copia copia) throws ReservaInvalidaException {
		if(copia != null){
			this.chave.setCopia(copia);
		}else{
			throw new ReservaInvalidaException(this.chave.getCliente(), copia, this.periodo);
		}
	}

	public Cliente getCliente() {
		return this.chave.getCliente();
	}

	public void setCliente(Cliente cliente) throws ReservaInvalidaException {
		if(cliente != null){
			this.chave.setCliente(cliente);
		}else{
			throw new ReservaInvalidaException(cliente, this.getCopia(), this.periodo);
		}
	}

	public Periodo getPeriodo() {
		return this.periodo;
	}

	public void setPeriodo(Periodo periodo) throws ReservaInvalidaException {
		if(periodo != null){
			this.periodo = periodo;
		}else{
			throw new ReservaInvalidaException(this.getCliente(), this.getCopia(), periodo);
		}
	}
	
	public ClienteCopiaPK getChave(){
		return this.chave;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Endereco && this.chave.equals(((Reserva)obj).getChave());
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
	
	@Override
	public String toString() {
		return String.format("Codigo cpia: %d    CPF Cliente: %d    %s", this.chave.getCopia().getCodigoCopia(), this.chave.getCliente().getCpf(), this.periodo.toString());
	}
}
