package dados.produto;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Table;

import dados.exception.AvaliacaoInvalidaException;
import dados.usuario.Usuario;

@Entity
@Table(name="AvaliacaoProduto")
public class AvaliacaoProduto implements Serializable {

	private static final long serialVersionUID = 7993080835551317752L;

	@EmbeddedId
	private AvaliacaoProdutoPK chave;
	
	@Column(name="id")
	@GeneratedValue
	private long id;
	
	@Column
	private Calendar data;

	@Column(updatable=true, name="Nota", nullable=false)
	private Integer nota;
	
	@Column(updatable=true, name="Recomendao", nullable=false)
	private Boolean recomendacao;

	@Column(updatable=true, name="Comentarios", nullable=false, length=1000)
	private String comentarios;
	
	public AvaliacaoProduto() {	}
	
	public AvaliacaoProduto(int nota, boolean recomendacao, String comentarios, Usuario usuario, Produto produto) throws AvaliacaoInvalidaException {
		if(AvaliacaoProduto.isValido(nota, recomendacao, comentarios)){
			this.nota = nota;
			this.recomendacao = recomendacao;
			this.comentarios = comentarios;
			this.chave = new AvaliacaoProdutoPK(usuario, produto);
		}else{
			throw new AvaliacaoInvalidaException();
		}
	}

	public static boolean isValido(int nota, boolean recomendacao, String comentarios){
		return nota >= 0 && nota <= 10 && comentarios != null && !comentarios.isEmpty(); 
	}

	public int getNota() {
		return this.nota;
	}

	public void setNota(int nota) throws AvaliacaoInvalidaException {
		if(AvaliacaoProduto.isValido(nota, this.recomendacao, this.comentarios)){
			this.nota = nota;
		}else{
			throw new AvaliacaoInvalidaException();
		}
	}

	public boolean isRecomendacao() {
		return this.recomendacao;
	}

	public void setRecomendacao(boolean recomendacao) throws AvaliacaoInvalidaException {
		if(AvaliacaoProduto.isValido(nota, this.recomendacao, this.comentarios)){
			this.recomendacao = recomendacao;
		}else{
			throw new AvaliacaoInvalidaException();
		}
	}

	public String getComentarios() {
		return this.comentarios;
	}

	public void setComentarios(String comentarios) throws AvaliacaoInvalidaException {
		if(AvaliacaoProduto.isValido(nota, this.recomendacao, this.comentarios)){
			this.comentarios = comentarios;
		}else{
			throw new AvaliacaoInvalidaException();
		}
	}

	public Usuario getUsuario() {
		return this.chave.getUsuario();
	}
	
	public Produto getProduto(){
		return this.chave.getProduto();
	}
	
	@Override
	public boolean equals(Object o) {
		return o instanceof AvaliacaoProduto && (this.getUsuario().equals(((AvaliacaoProduto)o).getUsuario()) || this.getProduto().equals(((AvaliacaoProduto)o).getProduto()));
	}
	
	@Override
	public String toString() {
		return String.format("Nota: %d", this.nota);
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public Calendar getData() {
		return data;
	}

	public void setData(Calendar data) {
		this.data = data;
	}

}
