package dados.produto;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import negocio.exception.LinkFotoExistenteException;
import util.IteratorBidirecional;

import dados.exception.ProdutoInvalidoException;

@Entity
@DiscriminatorValue("Documentario")
public class Documentario extends Produto {

	private static final long serialVersionUID = 2295325277253944522L;
	
	@Column(updatable=true, name="Tema", nullable=false)
	private String tema;
	
	public Documentario() {	}
	
	public Documentario(String nome, String genero, String censura, String descricao, String tema,
			String empresaProdutora, String linkTrailer, ClassificacaoPreco classificacao,
			Integer ano) throws ProdutoInvalidoException {
		super(nome, genero, censura, descricao, empresaProdutora, linkTrailer, classificacao, ano);
		
		if(tema != null){
			this.tema = tema.trim();
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getTema() {
		return this.tema;
	}

	public void setTema(String tema) throws ProdutoInvalidoException {
		if(tema != null){
			this.tema = tema;
		}else{
			throw new ProdutoInvalidoException();
		}
	}
	
	@Override
	public boolean equals(Object arg0) {
		return super.equals(arg0);
	}

	@Override
	public void setDados(Produto dados) throws ProdutoInvalidoException {
		if(dados != null && dados instanceof Documentario){
			Documentario d = (Documentario) dados;
			
			this.setAnoLancamento(d.getAnoLancamento());
			this.setCensura(d.getCensura());
			this.setNome(d.getNome());
			this.setClassificacaoPreco(d.getClassificacaoPreco());
			this.setDescricao(d.getDescricao());
			this.setEmpresaProdutora(d.getEmpresaProdutora());
			this.setGenero(d.getGenero());
			this.setLinkTrailer(d.getLinkTrailer());
			this.setTema(d.getTema());
			this.setNota(dados.getNota());
			
			IteratorBidirecional<LinkFoto> it = dados.getLinks();

			this.linkFotos.limpar();
			while(it.hasNext()){
				try {
					this.inserirLinkFoto(it.next());
				} catch (LinkFotoExistenteException e) {
				}
			}
		}else{
			throw new IllegalArgumentException();
		}
	}
}
