package dados.produto;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import negocio.exception.LinkFotoExistenteException;

import util.IteratorBidirecional;

import dados.exception.ProdutoInvalidoException;

@Entity
@DiscriminatorValue("Filme")
public class Filme extends Produto {

	private static final long serialVersionUID = -8482330183334008997L;

	@Column(updatable=true, name="Durao", nullable=false)
	private Integer duracao;

	@Column(updatable=true, name="Diretor", nullable=false)
	private String diretor;

	@Column(updatable=true, name="Autor", nullable=false)
	private String autor;

	public Filme() {}

	public Filme(String nome, String genero, String censura, String descricao,
			String empresaProdutora, String linkTrailer, String diretor, String autor,
			ClassificacaoPreco classificacao, Integer ano, Integer duracao) throws ProdutoInvalidoException {
		super(nome, genero, censura, descricao, empresaProdutora, linkTrailer, classificacao, ano);

		if(Filme.isValido(diretor, autor, ano)){
			this.diretor = diretor.trim();
			this.autor = autor.trim();
			this.duracao = duracao;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public static boolean isValido(String diretor, String autor, int duracao){
		return diretor != null && autor != null && duracao > 0;
	}

	public String getDiretor() {
		return this.diretor;
	}

	public void setDiretor(String diretor) throws ProdutoInvalidoException {
		if(Filme.isValido(diretor, this.autor, this.duracao)){
			this.diretor = diretor;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getAutor() {
		return this.autor;
	}

	public void setAutor(String autor) throws ProdutoInvalidoException {
		if(Filme.isValido(this.diretor, autor, this.duracao)){
			this.autor = autor;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public int getDuracao() {
		return this.duracao;
	}

	public void setDuracao(int duracao) throws ProdutoInvalidoException {
		if(Filme.isValido(this.diretor, this.autor, duracao)){
			this.duracao = duracao;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	@Override
	public boolean equals(Object arg0) {
		return super.equals(arg0);
	}

	@Override
	public void setDados(Produto dados) throws ProdutoInvalidoException {
		if(dados != null && dados instanceof Filme){
			Filme d = (Filme) dados;

			this.setAnoLancamento(d.getAnoLancamento());
			this.setCensura(d.getCensura());
			this.setClassificacaoPreco(d.getClassificacaoPreco());
			this.setDescricao(d.getDescricao());
			this.setNome(d.getNome());
			this.setEmpresaProdutora(d.getEmpresaProdutora());
			this.setGenero(d.getGenero());
			this.setLinkTrailer(d.getLinkTrailer());
			this.setAutor(d.getAutor());
			this.setDiretor(d.getDiretor());
			this.setDuracao(d.getDuracao());
			this.setNota(dados.getNota());
			IteratorBidirecional<LinkFoto> it = dados.getLinks();

			this.linkFotos.limpar();
			while(it.hasNext()){
				try {
					this.inserirLinkFoto(it.next());
				} catch (LinkFotoExistenteException e) {
				}
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

}
