package dados.produto;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import negocio.exception.LinkFotoExistenteException;
import util.IteratorBidirecional;

import dados.exception.ProdutoInvalidoException;

@Entity
@DiscriminatorValue("Jogo")
public class Jogo extends Produto {

	private static final long serialVersionUID = 5040791848347929912L;
	
	@Column(updatable=true, name="NumeroJogadores", nullable=false)
	private Integer quantidadeJogadores;

	@Column(updatable=true, name="Console", nullable=false)
	private String console;
	
	public Jogo() {	}
	
	public Jogo(String nome, String genero, String censura, String descricao, String videoGame,
			String empresaProdutora, String linkTrailer, ClassificacaoPreco classificacao,
			Integer ano, Integer quantidadeJogadores) throws ProdutoInvalidoException {
		super(nome, genero, censura, descricao, empresaProdutora, linkTrailer, classificacao, ano);
		
		if(Jogo.isValido(videoGame, quantidadeJogadores)){
			this.quantidadeJogadores = quantidadeJogadores;
			this.console = videoGame.trim();
		}else{
			throw new ProdutoInvalidoException();
		}
	}
	
	public static boolean isValido(String videoGame, int quantidadeJogadores){
		return videoGame != null && quantidadeJogadores > 0;
	}

	public int getQuantidadeJogadores() {
		return this.quantidadeJogadores;
	}

	public void setQuantidadeJogadores(int quantidadeJogadores) throws ProdutoInvalidoException {
		if(Jogo.isValido(this.console, quantidadeJogadores)){
			this.quantidadeJogadores = quantidadeJogadores;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getVideoGame() {
		return this.console;
	}

	public void setVideoGame(String videoGame) throws ProdutoInvalidoException {
		if(Jogo.isValido(videoGame, this.quantidadeJogadores)){
			this.console = videoGame.trim();
		}else{
			throw new ProdutoInvalidoException();
		}
	}
	
	@Override
	public boolean equals(Object arg0) {
		return super.equals(arg0);
	}

	@Override
	public void setDados(Produto dados) throws ProdutoInvalidoException {
		if(dados != null && dados instanceof Jogo){
			Jogo d = (Jogo) dados;
			
			this.setAnoLancamento(d.getAnoLancamento());
			this.setCensura(d.getCensura());
			this.setClassificacaoPreco(d.getClassificacaoPreco());
			this.setDescricao(d.getDescricao());
			this.setEmpresaProdutora(d.getEmpresaProdutora());
			this.setGenero(d.getGenero());
			this.setLinkTrailer(d.getLinkTrailer());
			this.setNome(d.getNome());
			this.setVideoGame(d.getVideoGame());
			this.setNota(dados.getNota());
			this.setQuantidadeJogadores(d.getQuantidadeJogadores());
			
			IteratorBidirecional<LinkFoto> it = dados.getLinks();

			this.linkFotos.limpar();
			while(it.hasNext()){
				try {
					this.inserirLinkFoto(it.next());
				} catch (LinkFotoExistenteException e) {
				}
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

}
