package dados.produto;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import dados.exception.LinkFotoInvalidoException;

@Entity
@Table(name="LinkFoto")
public class LinkFoto implements Serializable {
	
	private static final long serialVersionUID = 499618042290782434L;

	@Id @GeneratedValue
	private int id;
	
	@Column(name="Link")
	private String link;
	
	@Enumerated
	private TipoLinkFoto tipoLinkFoto;
	
	public LinkFoto() {}

	public LinkFoto(TipoLinkFoto tipo, String link) throws LinkFotoInvalidoException {
		if(tipo != null && LinkFoto.isValido(link)){
			this.tipoLinkFoto = tipo;
			this.link = link;		
		}else{
			throw new LinkFotoInvalidoException();
		}
	}
	
	public static boolean isValido(String link){
		boolean retorno = false;
		
		if(link != null && !link.isEmpty()){
			int i = 0, len = link.length();
			
			while(i < len){
				if(Character.isWhitespace(link.charAt(i))){
					i = len + 1;
				}else{
					i = i + 1;
				}
			}
			
			if(i == len){
				retorno = true;
			}
		}
		
		return retorno;
	}

	public TipoLinkFoto getTipoLinkFoto() {
		return this.tipoLinkFoto;
	}

	public void setLinkFoto(TipoLinkFoto tipo) throws LinkFotoInvalidoException {
		if(tipo != null){
			this.tipoLinkFoto = tipo;
		}else{
			throw new LinkFotoInvalidoException();
		}
	}

	public String getLink() {
		return this.link;
	}

	public void setLink(String link) throws LinkFotoInvalidoException {
		if(LinkFoto.isValido(link)){
			this.link = link;
		}else{
			throw new LinkFotoInvalidoException();
		}
	}

	@Override
	public boolean equals(Object o) {
		boolean retorno = o instanceof LinkFoto;
		
		if(retorno){
			LinkFoto l = (LinkFoto) o;
			
			retorno = this.link.equalsIgnoreCase(l.getLink());
		}
		
		return retorno;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}
}
