package dados.produto;

import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import negocio.exception.LinkFotoExistenteException;
import util.IteratorBidirecional;

import dados.exception.ProdutoInvalidoException;

@Entity
@DiscriminatorValue("Srie")
public class Serie extends Produto {
	
	private static final long serialVersionUID = 6680253386525693025L;
	
	@Column(updatable=true, name="Temporada", nullable=false)
	private Integer temporada;

	@Column(updatable=true, name="EpisodioInicial", nullable=false)
	private Integer episodioInicial;

	@Column(updatable=true, name="EpisodioFinal", nullable=false)
	private Integer episodioFinal;

	public Serie() {}

	public Serie(String nome, String genero, String censura, String descricao, String empresaProdutora,
			String linkTrailer, ClassificacaoPreco classificacao, Integer ano, Integer temporada, Integer episodioInicial, Integer episodioFinal) throws ProdutoInvalidoException {
		super(nome, genero, censura, descricao, empresaProdutora, linkTrailer, classificacao, ano);
		
		if(Serie.isValido(temporada, episodioInicial, episodioFinal)){
			this.temporada = temporada;
			this.episodioInicial = episodioInicial;
			this.episodioFinal = episodioFinal;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	private static boolean isValido(Integer temporada, Integer episodioInicial, Integer episodioFinal) {
		return temporada != null && temporada > 0 && episodioFinal != null && episodioInicial != null && episodioInicial > 0 && episodioFinal > 0 && episodioInicial <= episodioFinal;
	}

	public Integer getTemporada() {
		return this.temporada;
	}

	public void setTemporada(Integer temporada) throws ProdutoInvalidoException {
		if(Serie.isValido(temporada, this.episodioInicial, this.episodioFinal)){
			this.temporada = temporada;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public Integer getEpisodioInicial() {
		return this.episodioInicial;
	}

	public void setEpisodioInicial(Integer episodioInicial) throws ProdutoInvalidoException {
		if(Serie.isValido(this.temporada, episodioInicial, this.episodioFinal)){
			this.episodioInicial = episodioInicial;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public Integer getEpisodioFinal() {
		return this.episodioFinal;
	}

	public void setEpisodioFinal(Integer episodioFinal) throws ProdutoInvalidoException {
		if(Serie.isValido(this.temporada, this.episodioInicial, episodioFinal)){
			this.episodioFinal = episodioFinal;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	@Override
	public boolean equals(Object arg0) {
		return super.equals(arg0);
	}

	@Override
	public void setDados(Produto dados) throws ProdutoInvalidoException {
		if(dados != null && dados instanceof Serie){
			Serie d = (Serie) dados;
			
			this.setAnoLancamento(d.getAnoLancamento());
			this.setCensura(d.getCensura());
			this.setClassificacaoPreco(d.getClassificacaoPreco());
			this.setDescricao(d.getDescricao());
			this.setEmpresaProdutora(d.getEmpresaProdutora());
			this.setGenero(d.getGenero());
			this.setLinkTrailer(d.getLinkTrailer());
			this.setNome(d.getNome());
			this.setEpisodioFinal(d.getEpisodioFinal());
			this.setEpisodioInicial(d.getEpisodioInicial());
			this.setTemporada(d.getTemporada());
			this.setNota(dados.getNota());
			
			IteratorBidirecional<LinkFoto> it = dados.getLinks();

			this.linkFotos.limpar();
			while(it.hasNext()){
				try {
					this.inserirLinkFoto(it.next());
				} catch (LinkFotoExistenteException e) {
				}
			}

		}else{
			throw new IllegalArgumentException();
		}
	}
}
