package dados.usuario;

import javax.persistence.CascadeType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;

import negocio.exception.SenhaInvalidaException;
import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.NomeInvalidoException;

@Entity
@DiscriminatorValue("Dependente")
public class Dependente extends Usuario {

	private static final long serialVersionUID = -2193252213014002778L;

	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, optional=false, fetch=FetchType.EAGER)
	@JoinColumn(name="id_Cliente")
	private Cliente cliente;

	public Dependente() {}

	public Dependente(long cpf, String senha, String nome, String email, Cliente cliente) throws NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, EmailInvalidoException {
		super(cpf, senha, nome, email);

		this.setCliente(cliente);
	}

	public Cliente getCliente() {
		return this.cliente;
	}

	public void setCliente(Cliente cliente) {
		if(cliente != null){
			this.cliente = cliente;
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoCadastro(Usuario user) {
		if(user != null){
			return false;
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoRemocao(Usuario user) {
		if(user != null){
			return this.equals(user);
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public void setDados(Usuario usuario) {
		if(usuario != null && usuario instanceof Dependente){
			Dependente d = (Dependente) usuario;

			try {
				this.setSenha(d.getSenha());
				this.setEmail(d.getEmail());
				this.setNome(d.getNome());
			} catch (SenhaInvalidaException e) {
				// NAO deve acontecer
			} catch (NomeInvalidoException e) {
				// NAO deve acontecer
			} catch (EmailInvalidoException e) {
				// NAO deve acontecer
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoAlteracao(Usuario user) {
		if(user != null){
			return this.equals(user);
		}else{
			throw new IllegalArgumentException();
		}
	}

}
