package dados.usuario;

import javax.persistence.CascadeType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;

import negocio.exception.SenhaInvalidaException;

import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.FilialInvalidaException;
import dados.exception.NomeInvalidoException;
import dados.filial.Filial;

@Entity
@DiscriminatorValue("Funcionario")
public class Funcionario extends Usuario{

	private static final long serialVersionUID = 8364553045210684685L;

	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, fetch = FetchType.EAGER, optional = false)
	@JoinColumn(name = "id_Filial")
	private Filial filial;

	public Funcionario() {}

	public Funcionario(long cpf, String senha, String nome, String email, Filial filial) throws FilialInvalidaException, NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, EmailInvalidoException{
		super(cpf, senha, nome, email);

		if(Funcionario.isValido(filial)){
			this.filial = filial;
		}else{
			throw new FilialInvalidaException();
		}
	}

	public static boolean isValido(Filial filial){
		return filial != null;
	}

	public Filial getFilial(){
		return this.filial;
	}

	public void setSalario(Filial filial) throws FilialInvalidaException {
		if(filial != null){
			this.filial = filial;
		}else{
			throw new FilialInvalidaException();
		}
	}

	@Override
	public boolean temPermissaoCadastro(Usuario user) {
		if(user != null){
			return !(user instanceof Administrador) && !(user instanceof Funcionario);
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoRemocao(Usuario user) {
		if(user != null){
			return user.equals(this) || user instanceof Cliente || user instanceof Dependente;
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public void setDados(Usuario usuario) {
		if(usuario != null && usuario instanceof Funcionario){
			Funcionario f = (Funcionario) usuario;

			try {
				this.setSenha(f.getSenha());
				this.setEmail(f.getEmail());
				this.setNome(f.getNome());
				this.setSalario(f.getFilial());
				this.filial = f.getFilial();
			} catch (SenhaInvalidaException e) {
				// NAO deve ocorrer
			} catch (EmailInvalidoException e) {
				// NAO deve ocorrer
			} catch (NomeInvalidoException e) {
				// NAO deve ocorrer
			} catch (FilialInvalidaException e) {
				// NAO deve ocorrer
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoAlteracao(Usuario user) {
		return this.temPermissaoRemocao(user);
	}

}
