package dados.usuario;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import dados.exception.PlanoInvalidoException;
import dados.produto.LinkFoto;

@Entity
@Table(name="Plano")
public class Plano implements Serializable{
	
	private static final long serialVersionUID = 9081878317054683430L;

	@Id
	@Column(name="id_Plano")
	private String nome;
	
	@Column(name="id")
	@GeneratedValue
	private long id;
	
	@Column(updatable=true, name="Mensalidade", nullable=false)
	private Double mensalidade;
	
	@Column(updatable=true, name="Descrio", nullable=false, length=1000)
	private String descricao;
	
	@Column(updatable=true, name="Locaes", nullable=false)
	private Integer locacoesGratuitas;
	
	@Enumerated
	private LinkFoto foto;
	
	public Plano() {}
	
	public Plano(Double mensalidade, String nome, String descricao, int locacoesMensais, LinkFoto foto) throws PlanoInvalidoException {
		if(Plano.isValido(mensalidade, nome, descricao, locacoesMensais, foto)){
			this.descricao = descricao.trim();
			this.mensalidade = mensalidade;
			this.nome = nome.trim();
			this.locacoesGratuitas = locacoesMensais;
			this.foto = foto;
		}else{
			throw new PlanoInvalidoException();
		}
	}
	
	public static boolean isValido(Double mensalidade, String nome, String descricao, int locacoesMensais, LinkFoto foto) {
		return mensalidade != null && mensalidade >= 0.0 && nome != null && descricao != null && locacoesMensais >= 0 && foto != null;
	}
	
	public Double getMensalidade() {
		return this.mensalidade;
	}

	public void setMensalidade(double mensalidade) throws PlanoInvalidoException {
		if(Plano.isValido(mensalidade, this.nome, this.descricao, this.locacoesGratuitas, this.foto)){
			this.mensalidade = mensalidade;
		}else{
			throw new PlanoInvalidoException();
		}
	}

	public String getNome() {
		return this.nome;
	}

	public void setNome(String nome) throws PlanoInvalidoException {
		if(Plano.isValido(this.mensalidade, nome, this.descricao, this.locacoesGratuitas, this.foto)){
			this.nome = nome;
		}else{
			throw new PlanoInvalidoException();
		}
	}

	public String getDescricao() {
		return this.descricao;
	}

	public void setDescricao(String descricao) throws PlanoInvalidoException {
		if(Plano.isValido(this.mensalidade, this.nome, descricao, this.locacoesGratuitas, this.foto)){
			this.descricao = descricao;
		}else{
			throw new PlanoInvalidoException();
		}
	}
	
	public void setLocacoesGratuitas(int locacoesGratuitas) {
		if(locacoesGratuitas >= 0){
			this.locacoesGratuitas = locacoesGratuitas;
		}else{
			throw new IllegalArgumentException();
		}
	}

	public int getLocacoesGratuitas() {
		return this.locacoesGratuitas;
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Plano && this.nome.equals(((Plano)obj).getNome());
	}

	@Override
	public String toString() {
		return String.format("%s - R$ %.2f", this.nome, this.mensalidade, this.foto);
	}

	public LinkFoto getFoto() {
		return this.foto;
	}

	public void setFoto(LinkFoto foto) {
		this.foto = foto;
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}
}
