package dados.usuario;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import dados.exception.TelefoneInvalidoException;

@Entity
@Table(name="Telefone")
public class Telefone implements Serializable {

	private static final long serialVersionUID = 3268968721318221771L;

	@Id @GeneratedValue()
	@Column(name="id_Telefone")
	private Long id;
	
	@Column(updatable=true, name="DDD", nullable=false)
	private Integer ddd;

	@Column(updatable=true, name="Numero", nullable=false)
	private Integer numero;
	
	public Telefone() {}
	
	public Telefone(Long id, int ddd, int numero) throws TelefoneInvalidoException {
		if(id != null && Telefone.isValido(ddd, numero)){
			this.id = id;
			this.ddd = ddd;
			this.numero = numero;
		}else{
			throw new TelefoneInvalidoException();
		}
	}
	
	public Telefone(int ddd, int numero) throws TelefoneInvalidoException {
		if(Telefone.isValido(ddd, numero)){
			this.ddd = ddd;
			this.numero = numero;
		}else{
			throw new TelefoneInvalidoException();
		}
	}

	public static boolean isValido(int ddd, int numero) {
		return ddd > 0 && ddd <= 999 && numero >= 10000000 && numero <= 99999999;
	}

	public Integer getDDD() {
		return this.ddd;
	}
	
	public Long getID(){
		return this.id;
	}
	
	public void setID(Long id) throws TelefoneInvalidoException{
		if(id != null){
			this.id = id;
		}else{
			throw new TelefoneInvalidoException();
		}
	}
	
	public void setDDD(int ddd) throws TelefoneInvalidoException {
		if(Telefone.isValido(ddd, this.numero)){
			this.ddd = ddd;
		}else{
			throw new TelefoneInvalidoException();
		}
	}

	public int getNumero() {
		return this.numero;
	}

	public void setNumero(int numero) throws TelefoneInvalidoException {
		if(Telefone.isValido(this.ddd, numero)){
			this.numero = numero;
		}else{
			throw new TelefoneInvalidoException();
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		return obj instanceof Telefone && (this.id == ((Telefone)obj).getID() || (this.ddd.equals(((Telefone)obj).getDDD()) && this.numero.equals(((Telefone)obj).getNumero())));
	}
	
	@Override
	public String toString() {
		return String.format("%d-%d", this.ddd, this.numero);
	}

}
