package negocio;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Embeddable;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;

import util.IteratorBidirecional;

import dados.usuario.Telefone;

@Embeddable
public class GerenciadorTelefone implements Serializable{

	private static final long serialVersionUID = 1L;

	@OneToMany(cascade = CascadeType.ALL, fetch=FetchType.EAGER)
	private Set<Telefone> telefones;
	
	public GerenciadorTelefone() {
		this.telefones = new HashSet<Telefone>();
	}
	
	public void inserirTelefone(Telefone telefone){
		if(telefone != null){
			if(!this.telefones.contains(telefone)){
				this.telefones.add(telefone);
			}
		}else{
			throw new IllegalArgumentException();
		}
	}
	
	public void removerTelefone(Telefone telefone) {
		if(telefone != null){
			if(this.telefones.contains(telefone)){
				this.telefones.remove(telefone);
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	public IteratorBidirecional<Telefone> getTelefones() {
		return new IteratorBidirecional<Telefone>(this.telefones.iterator());
	}

	public void setTelefones(Telefone[] telefones2){
		if(telefones2 != null){
			this.telefones.clear();
			
			for(int i = 0; i < telefones2.length; i++){
				this.inserirTelefone(telefones2[i]);
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	public void limpar() {
		this.telefones.clear();
	}

}
