package negocio.produto;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Embeddable;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;

import util.IteratorBidirecional;

import negocio.exception.LinkFotoExistenteException;
import negocio.exception.LinkInexistenteException;

import dados.produto.LinkFoto;
import dados.produto.TipoLinkFoto;

@Embeddable
public class GerenciadorLinkFotos implements Serializable{

	private static final long serialVersionUID = 1L;
	
	@OneToMany(cascade = CascadeType.ALL, fetch=FetchType.EAGER)
	private Set<LinkFoto> linkFotos;
	
	public GerenciadorLinkFotos() {
		this.linkFotos = new HashSet<LinkFoto>();
	}
	
	public void inserirLink(LinkFoto link) throws LinkFotoExistenteException{
		if(link != null){
			if(!this.linkFotos.contains(link)){
				this.linkFotos.add(link);
			}else{
				throw new LinkFotoExistenteException(link);
			}
		}else{
			throw new IllegalArgumentException();
		}
	}
	
	public void removerLink(String link) throws LinkInexistenteException {
		if(link != null){
			IteratorBidirecional<LinkFoto> it = this.getLinks();
			LinkFoto c = null;
			
			while(c == null && it.hasNext()){
				c = it.next();
				
				if(!c.getLink().equals(link)){
					c = null;
				}
			}
			
			if(c != null){
				this.linkFotos.remove(c);
			}else{
				throw new LinkInexistenteException(link);
			}
		}else{
			throw new IllegalArgumentException();
		}
	}
	
	public IteratorBidirecional<LinkFoto> getLinks(){
		return new IteratorBidirecional<LinkFoto>(this.linkFotos.iterator());
	}
	
	public LinkFoto getLinkPoster(){
		IteratorBidirecional<LinkFoto> it = this.getLinks();
		LinkFoto retorno = null;
		
		while(it.hasNext() && retorno == null){
			retorno = it.next();
			
			if(retorno.getTipoLinkFoto() != TipoLinkFoto.POSTER){
				retorno = null;
			}
		}
		
		return retorno;
	}
	
	public LinkFoto getLinkBox(){
		IteratorBidirecional<LinkFoto> it = this.getLinks();
		LinkFoto retorno = null;
		
		while(it.hasNext() && retorno == null){
			retorno = it.next();
			
			if(retorno.getTipoLinkFoto() != TipoLinkFoto.FOTOBOX){
				retorno = null;
			}
		}
		
		if(retorno == null){
			retorno = this.getLinkPoster();
		}
		return retorno;
	}

	public void limpar() {
		this.linkFotos.clear();
	}
}
