package servicos;

import java.rmi.Remote;
import java.rmi.RemoteException;

import util.IteratorBidirecional;

import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.FilialInexistenteException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.LocacaoExistenteException;
import negocio.exception.LocacaoInexistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.PromocaoInexistenteException;
import negocio.exception.ReservaExistenteException;
import negocio.exception.ReservaInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioLogadoException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import dados.exception.FilialInvalidaException;
import dados.exception.LocacaoInvalidaException;
import dados.locacao.Copia;
import dados.locacao.Locacao;
import dados.locacao.Periodo;
import dados.locacao.Promocao;
import dados.locacao.Reserva;
import dados.produto.CriterioBusca;
import dados.produto.Produto;
import dados.produto.TipoProduto;
import dados.usuario.Cliente;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;

public interface InterfaceGeral extends Remote{
	
	public final String ip = "localhost";
	
	/**
	 * Este mtodo loga o usurio de CPF e senha fornecidos, retornando uma sesso
	 * para uso do mesmo.
	 * @param cpf do usuario
	 * @param senha do usuario
	 * @return sessao de login
	 * @throws UsuarioInexistenteException 
	 * @throws UsuarioLogadoException 
	 * @throws SenhaInvalidaException 
	 */
	public abstract long logarUsuario(long cpf, String senha) throws RemoteException, UsuarioLogadoException, UsuarioInexistenteException, SenhaInvalidaException;
	
	/**
	 * Este mtodo desloga o usurio de sesso recebida como parmetro.
	 * @param sessao a ser removida
	 * @throws UsuarioNaoLogadoException 
	 */
	public abstract void deslogar(long sessao) throws RemoteException, UsuarioNaoLogadoException;
	
	/**
	 * Este mtodo cadastra o usurio recebido como parmetro.
	 * @param sessao
	 * @param usuario a ser cadastrado
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioInvalidoException 
	 * @throws UsuarioExistenteException 
	 * @throws UsuarioSemPermissaoException 
	 */
	public abstract void cadastrarUsuario(long sessao, Usuario usuario) throws RemoteException, UsuarioSemPermissaoException, UsuarioExistenteException, UsuarioNaoLogadoException;
	
	public abstract Usuario buscarUsuario(long sessao, long cpf) throws RemoteException, UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException;
	
	public abstract void alterarDadosUsuario(long sessao, Usuario dados) throws RemoteException, UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioExistenteException, UsuarioSemPermissaoException;
	
	/**
	 * Este mtodo descadastra o usurio recebido como parmetro.
	 * @param sessao
	 * @param cpf do Usuario a ser removido
	 * @throws SenhaInvalidaException 
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioSemPermissaoException 
	 * @throws UsuarioInexistenteException 
	 */
	public abstract void descadastrarUsuario(long sessao, long cpfUsuario, String senha) throws RemoteException, UsuarioInexistenteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, SenhaInvalidaException;
	
	public abstract void alterarTelefonesCliente(long sessao, long cpf, Telefone[] telefones) throws RemoteException, UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException;
	
	public abstract Plano buscarPlano(String nome) throws RemoteException, PlanoInexistenteException;
	
	/**
	 * Este mtodo retorna todos os planos disponiveis pela rede de locadoras
	 * @return planos da locadora
	 * @throws PlanoInexistenteException 
	 */
	public abstract IteratorBidirecional<Plano> getPlanos() throws RemoteException, PlanoInexistenteException;
	
	public abstract void inserirLocacao(long sessao, Locacao locacao) throws RemoteException, UsuarioNaoLogadoException, LocacaoExistenteException, LocacaoInvalidaException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException;
	
	public abstract Locacao buscarLocacao(long sessao, Copia copia) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, LocacaoInexistenteException;
	
	public abstract IteratorBidirecional<Locacao> buscarLocacoes(long sessao, Cliente cliente) throws RemoteException, LocacaoInexistenteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException;
	
	public abstract IteratorBidirecional<Locacao> buscarLocacoesInterceptaPeriodo(Copia copia, Periodo periodo) throws LocacaoInexistenteException, RemoteException;
	
	public abstract void removerLocacao(long sessao, Copia copia) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, LocacaoInexistenteException;
	
	public abstract Promocao buscarPromocao(String nome) throws RemoteException, PromocaoInexistenteException;
	
	public abstract IteratorBidirecional<Promocao> getPromocoes() throws RemoteException, PromocaoInexistenteException;
	
	public abstract void inserirReserva(long sessao, Reserva reserva) throws RemoteException, UsuarioNaoLogadoException, ReservaExistenteException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException, FilialNaoLogadaException, FilialInexistenteException;
	
	public abstract Reserva buscarReserva(long sessao, Cliente cliente, Copia copia) throws RemoteException, UsuarioNaoLogadoException, ReservaInexistenteException, UsuarioSemPermissaoException;
	
	public abstract IteratorBidirecional<Reserva> buscarReservas(long sessao, Copia copia) throws RemoteException, UsuarioNaoLogadoException, ReservaInexistenteException, UsuarioSemPermissaoException;
	
	public abstract IteratorBidirecional<Reserva> buscarReservas(long sessao, Cliente cliente) throws RemoteException, ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException;
	
	public abstract IteratorBidirecional<Reserva> buscarReservasInterceptaPeriodo(long sessao, Copia copia, Periodo periodo) throws ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, RemoteException;
	
	public abstract void removerReserva(long sessao, long cpf, int codigoCopia) throws RemoteException, ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, UsuarioInexistenteException, CopiaInexistenteException, ProdutoInexistenteException;
	
	/**
	 * Este mtodo retorna um produto com nome e gnero fornecidos.
	 * @param sessao
	 * @param nome do produto
	 * @param genero do produto
	 * @return produto
	 * @throws ProdutoInexistenteException 
	 */
	public abstract Produto buscarProduto(String nomeProduto) throws RemoteException, ProdutoInexistenteException;
	
	public abstract IteratorBidirecional<Produto> buscarProdutoTipo(TipoProduto tipo) throws RemoteException, ProdutoInexistenteException;

	public abstract IteratorBidirecional<Produto> buscarProdutoRestricoes(CriterioBusca restricoes) throws RemoteException, ProdutoInexistenteException;
	
	public abstract IteratorBidirecional<Produto> getProdutos() throws RemoteException, ProdutoInexistenteException;
	
	public abstract double devolverProduto(long sessao, int codigoCopia, String filial) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, LocacaoInexistenteException, ProdutoInexistenteException, CopiaInexistenteException, FilialInvalidaException, RemoteException, FilialInexistenteException, UsuarioInexistenteException;
	
	public abstract Copia buscarCopia(int codigoCopia) throws CopiaInexistenteException, RemoteException;

	public abstract Plano buscarPlano(long id) throws PlanoInexistenteException, RemoteException;

	public abstract Locacao buscarLocacao(long id) throws LocacaoInexistenteException,RemoteException;

	public abstract Promocao buscarPromocao(long id) throws PromocaoInexistenteException, RemoteException;

	public abstract Reserva buscaReserva(long id) throws ReservaInexistenteException, RemoteException;

	public abstract Produto buscarProduto(long id) throws ProdutoInexistenteException, RemoteException;

	public abstract Copia getCopiaFilial(String nomeProduto, String nome) throws RemoteException, CopiaInexistenteException, ProdutoInexistenteException;
	
}
