package util;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Esta classe representa um {@link Iterator} que implementa a interface
 * {@link Serializable} para poder ser utilizado por RMI. Alm disso, permite uma
 * navegao sobre a coleo de forma bidirecional, como uma lista duplamente
 * ligada.
 * @see Iterator
 * @author Caio Csar
 * @param <T> - Tipo do objeto a ser iterado.
 */
public class IteratorBidirecional<T> implements Iterator<T>, Serializable{

	private static final long serialVersionUID = 8873105054199381296L;
	
	private Object[] objetos;
	private int proxima;
	private int size;
	
	public IteratorBidirecional(Iterator<T> iterador) {
		List<T> lista = new Vector<T>();
		
		while(iterador.hasNext()){
			lista.add(iterador.next());
		}
		
		this.objetos = lista.toArray();
		this.size = this.objetos.length;
	}
	
	public IteratorBidirecional(Collection<T> iterador) {
		List<T> lista = new Vector<T>();

		for(T elemento : iterador){
			lista.add(elemento);
		}
		
		this.objetos = lista.toArray();
		this.size = this.objetos.length;
	}
	
	@Override
	public boolean hasNext() {
		return this.proxima < this.size && this.objetos[this.proxima] != null;
	}

	public boolean hasPrevious(){
		return this.proxima > 0;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public T next() {
		if(this.hasNext()){
			T t = (T) this.objetos[this.proxima];
			this.proxima = this.proxima + 1;
			return t;
		}else{
			throw new IllegalStateException();
		}
	}
	
	@SuppressWarnings("unchecked")
	public T previous() {
		if(this.hasPrevious()){
			this.proxima = this.proxima - 1;
			return (T)this.objetos[this.proxima];
		}else{
			throw new IllegalStateException();
		}
	}

	@Override
	public void remove() {
		if(this.hasPrevious()){
			this.objetos[this.proxima - 1] = null;
			this.proxima = this.proxima - 1;
			
			int i = this.proxima;
			while(i < this.size - 1){
				this.objetos[i] = this.objetos[i + 1];
				i = i + 1;
			}
			
			this.objetos[i] = null;
		}else{
			throw new IllegalStateException();
		}
	}

	public static void main(String[] args) {
		
	}
}
